classdef KeaCamera < tof.LiveCamera
  % KeaCamera
  %   The main interface to the kea camera
  %
  % Methods:
  %   onCameraProcessingCapable - Check whether this kea camera can do on camera
  %                               processing
  %   getOnCameraProcessing - Get on camera processing
  %   setOnCameraProcessing - Set on camera processing
  %   setCameraConfig - Set the camera config
  %   setConfigurations - Set both the camera and processing config
  %   multipleConfigurationsPossible - Check whether camera supports multiple
  %                                    configurations
  %   setMultipleConfigurations - Set multiple configurations
  %   setProcessConfig - the processing config of the camera
  %

  methods(Access=public)
    function obj = KeaCamera(varargin)
      % KeaCamera
      %   Construct new kea_camera class
      %
      % ----------
      %
      %   Construct the kea camera with the given processing and camera config.
      %
      % Inputs:
      %   1 - string - The serial number of the camera to connect to, if the
      %                camera is available over both usb and gige it will be
      %                connected over usb. If an empty string is supplied the
      %                camera will connect to the first usb camera, if no usb
      %                camera is connected it will connect to the first gige
      %                camera that replies (this reply time is random, so you
      %                might connect to a different camera each time if you've
      %                got multiple cameras connected to your network). If no
      %                camera is found with the given serial number a nullptr
      %                and error is returned.
      %   2 - ProcessingConfig - The processing config
      %   3 - CameraConfig - The camera config
      %
      % ----------
      %
      %   Construct the kea camera
      %
      % Inputs:
      %   1 - string - The serial number of the camera to connect to, if the
      %                camera is available over both usb and gige it will be
      %                connected over usb. If an empty string is supplied the
      %                camera will connect to the first usb camera, if no usb
      %                camera is connected it will connect to the first gige
      %                camera that replies (this reply time is random, so you
      %                might connect to a different camera each time if you've
      %                got multiple cameras connected to your network). If no
      %                camera is found with the given serial number a nullptr
      %                and error is returned.
      %   2 - ProcessingConfig - The processing config
      %
      % ----------
      %
      %   Construct the kea camera with a processing config generated by
      %   default_processing from the camera config
      %
      % Inputs:
      %   1 - string - The serial number of the camera to connect to, if the
      %                camera is available over both usb and gige it will be
      %                connected over usb. If an empty string is supplied the
      %                camera will connect to the first usb camera, if no usb
      %                camera is connected it will connect to the first gige
      %                camera that replies (this reply time is random, so you
      %                might connect to a different camera each time if you've
      %                got multiple cameras connected to your network). If no
      %                camera is found with the given serial number a nullptr
      %                and error is returned.
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        ptr = varargin{1};
      elseif nargin == 3 && (isStringScalar(varargin{1}) || ischar(varargin{1})) && isa(varargin{2}, 'tof.ProcessingConfig') && isa(varargin{3}, 'tof.CameraConfig')
        ptr = tof.mex('tof_kea_camera_new_both_configs', varargin{1}, varargin{2}.get_object_ptr(), varargin{3}.get_object_ptr());
      elseif nargin == 2 && (isStringScalar(varargin{1}) || ischar(varargin{1})) && isa(varargin{2}, 'tof.ProcessingConfig')
        ptr = tof.mex('tof_kea_camera_new', varargin{1}, varargin{2}.get_object_ptr());
      elseif nargin == 1 && (isStringScalar(varargin{1}) || ischar(varargin{1}))
        ptr = tof.mex('tof_kea_camera_new_simple', varargin{1});
      else
        error("Can't construct empty object");
      end

      obj@tof.LiveCamera(ptr);

    end

    function delete(~)
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = onCameraProcessingCapable(obj)
      % onCameraProcessingCapable
      %   Check whether this kea camera can do on camera processing
      %
      % Output:
      %   ret_val - logical - On camera processing capable
      %

      ret_val = tof.mex('tof_kea_camera_on_camera_processing_capable', obj.mPtr);
    end

    function ret_val = getOnCameraProcessing(obj)
      % getOnCameraProcessing
      %   Get on camera processing
      %
      % Output:
      %   ret_val - logical - True on camera processing, false on host
      %                       processing
      %

      ret_val = tof.mex('tof_kea_camera_get_on_camera_processing', obj.mPtr);
    end

    function setOnCameraProcessing(obj, on_camera_processing)
      % setOnCameraProcessing
      %   Set on camera processing
      %
      % Inputs:
      %   on_camera_processing - logical - True on camera processing, false on
      %                                    host processing
      %

      tof.mex('tof_kea_camera_set_on_camera_processing', obj.mPtr, on_camera_processing);
    end

    function setCameraConfig(obj, camera_config)
      % setCameraConfig
      %   Set the camera config
      %
      % Inputs:
      %   camera_config - CameraConfig - Camera config
      %

      tof.mex('tof_kea_camera_set_camera_config', obj.mPtr, camera_config.get_object_ptr());
    end

    function setConfigurations(obj, camera_config, processing_config)
      % setConfigurations
      %   Set both the camera and processing config
      %
      % Inputs:
      %   camera_config - CameraConfig - Camera config
      %   processing_config - ProcessingConfig - Processing config
      %

      tof.mex('tof_kea_camera_set_configurations', obj.mPtr, camera_config.get_object_ptr(), processing_config.get_object_ptr());
    end

    function ret_val = multipleConfigurationsPossible(obj)
      % multipleConfigurationsPossible
      %   Check whether camera supports multiple configurations
      %
      % Output:
      %   ret_val - logical - Multiple configurations possible
      %

      ret_val = tof.mex('tof_kea_camera_multiple_configurations_possible', obj.mPtr);
    end

    function setMultipleConfigurations(obj, camera_configs, pro_configs)
      % setMultipleConfigurations
      %   Set multiple configurations
      %
      % Inputs:
      %   camera_configs - vector[CameraConfig] - Camera configurations
      %   pro_configs - vector[ProcessingConfig] - Processing configurations
      %

      tof.mex('tof_kea_camera_set_multiple_configurations', obj.mPtr, arrayfun(@(vec_obj)vec_obj.get_object_ptr(), camera_configs), arrayfun(@(vec_obj)vec_obj.get_object_ptr(), pro_configs));
    end

    function setProcessConfig(obj, config)
      % setProcessConfig
      %   the processing config of the camera
      %
      % Inputs:
      %   config - ProcessingConfig - The processing config
      %

      tof.mex('tof_kea_camera_set_process_config', obj.mPtr, config.get_object_ptr());
    end

  end
end
