classdef LiveCamera < tof.Camera
  % LiveCamera
  %   Functions shared across all live cameras
  %
  % Methods:
  %   version - Get the tof library version running on the camera
  %   remoteUpdateCapable - Check whether the camera is capable of updating
  %                         remotely
  %   remoteUpdate - Update the camera remotely. After uploading the update file
  %                  the function will return. If a correct update file was
  %                  uploaded, the camera will flash the update and reboot. This
  %                  generally takes about a minute. The camera can also be
  %                  updated via a web interface by visiting the ip address of
  %                  the camera on port 8080, for example
  %                  http://192.168.1.208:8080
  %   getEeprom - Get the eeprom located on the camera
  %   setEeprom - Set the eeprom located on the camera
  %   persistentIpCapable - Whether the camera is capable of setting persistent
  %                         ip
  %   getPersistentIp - Get the current persistent ip settings
  %   setPersistentIp - Set the persistent ip settings. To activate persistent
  %                     ip settings run apply_persistent_ip. 
  %   applyPersistentIpCapable - Whether the camera is capable of applying the
  %                              persistent ip
  %   applyPersistentIp - Apply persistent ip by rebooting the camera, power
  %                       cycling the camera is discouraged because the applied
  %                       ip settings might not stick.
  %   softwareTriggerCapable - Check whether the camera is capable of software
  %                            trigger
  %   softwareTrigger - Software trigger the camera
  %   getDelay - Get delay between network packets
  %   setDelay - Set delay between network packets
  %   getPacketSize - Get maximum size of each network packet transmitted
  %   setPacketSize - Set maximum size of each network packet transmitted
  %   runProcess - run command on camera
  %   uploadFile - Upload file to camera
  %   downloadFile - Download file from camera
  %   getTemperaturesCapable - Whether the camera is capable of returning
  %                            temperatures
  %   getTemperatures - Get the current on camera temperatures
  %   getGpio - Get voltage level on given pin
  %   setGpio - Set voltage level on given pin
  %

  methods(Access=public)
    function obj = LiveCamera(varargin)
      % LiveCamera
      %   Construct new live_camera class
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        ptr = varargin{1};
      else
        error("Can't construct empty object");
      end

      obj@tof.Camera(ptr);

    end

    function delete(~)
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = version(obj)
      % version
      %   Get the tof library version running on the camera
      %
      % Output:
      %   ret_val - string - Version
      %

      ret_val = tof.mex('tof_live_camera_version', obj.mPtr);
    end

    function ret_val = remoteUpdateCapable(obj)
      % remoteUpdateCapable
      %   Check whether the camera is capable of updating remotely
      %
      % Output:
      %   ret_val - logical - Remote update capable
      %

      ret_val = tof.mex('tof_live_camera_remote_update_capable', obj.mPtr);
    end

    function remoteUpdate(obj, file_location)
      % remoteUpdate
      %   Update the camera remotely. After uploading the update file the
      %   function will return. If a correct update file was uploaded, the
      %   camera will flash the update and reboot. This generally takes about a
      %   minute. The camera can also be updated via a web interface by visiting
      %   the ip address of the camera on port 8080, for example
      %   http://192.168.1.208:8080
      %
      % Inputs:
      %   file_location - string - Location of the update file
      %

      tof.mex('tof_live_camera_remote_update', obj.mPtr, file_location);
    end

    function ret_val = persistentIpCapable(obj)
      % persistentIpCapable
      %   Whether the camera is capable of setting persistent ip
      %
      % Output:
      %   ret_val - logical - Persistent ip capable
      %

      ret_val = tof.mex('tof_live_camera_persistent_ip_capable', obj.mPtr);
    end

    function ret_val = getPersistentIp(obj)
      % getPersistentIp
      %   Get the current persistent ip settings
      %
      % Output:
      %   ret_val - PersistentIp - Persistent IP
      %

      ret_val = tof.mex('tof_live_camera_get_persistent_ip', obj.mPtr);
      ret_val = tof.PersistentIp(ret_val);
    end

    function setPersistentIp(obj, persistent_ip)
      % setPersistentIp
      %   Set the persistent ip settings. To activate persistent ip settings run
      %   apply_persistent_ip. 
      %
      % Inputs:
      %   persistent_ip - PersistentIp - The persistent ip
      %

      tof.mex('tof_live_camera_set_persistent_ip', obj.mPtr, persistent_ip.get_object_ptr());
    end

    function ret_val = applyPersistentIpCapable(obj)
      % applyPersistentIpCapable
      %   Whether the camera is capable of applying the persistent ip
      %
      % Output:
      %   ret_val - logical - Capable
      %

      ret_val = tof.mex('tof_live_camera_apply_persistent_ip_capable', obj.mPtr);
    end

    function applyPersistentIp(obj)
      % applyPersistentIp
      %   Apply persistent ip by rebooting the camera, power cycling the camera
      %   is discouraged because the applied ip settings might not stick.
      %

      tof.mex('tof_live_camera_apply_persistent_ip', obj.mPtr);
    end

    function ret_val = softwareTriggerCapable(obj)
      % softwareTriggerCapable
      %   Check whether the camera is capable of software trigger
      %
      % Output:
      %   ret_val - logical - Software trigger capable
      %

      ret_val = tof.mex('tof_live_camera_software_trigger_capable', obj.mPtr);
    end

    function softwareTrigger(obj)
      % softwareTrigger
      %   Software trigger the camera
      %

      tof.mex('tof_live_camera_software_trigger', obj.mPtr);
    end

    function ret_val = getDelay(obj)
      % getDelay
      %   Get delay between network packets
      %
      % Output:
      %   ret_val - numeric - The delay, depending on the camera software it is
      %                       in nanoseconds or in cpu instruction
      %

      ret_val = tof.mex('tof_live_camera_get_delay', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setDelay(obj, delay)
      % setDelay
      %   Set delay between network packets
      %
      % Inputs:
      %   delay - numeric - The delay, depending on the camera software it is in
      %                     nanoseconds or in cpu instruction
      %

      tof.mex('tof_live_camera_set_delay', obj.mPtr, delay);
    end

    function ret_val = getPacketSize(obj)
      % getPacketSize
      %   Get maximum size of each network packet transmitted
      %
      % Output:
      %   ret_val - numeric - Packet size in bytes
      %

      ret_val = tof.mex('tof_live_camera_get_packet_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setPacketSize(obj, size)
      % setPacketSize
      %   Set maximum size of each network packet transmitted
      %
      % Inputs:
      %   size - numeric - Packet size in bytes
      %

      tof.mex('tof_live_camera_set_packet_size', obj.mPtr, size);
    end

    function ret_val = getTemperaturesCapable(obj)
      % getTemperaturesCapable
      %   Whether the camera is capable of returning temperatures
      %
      % Output:
      %   ret_val - logical - Capable
      %

      ret_val = tof.mex('tof_live_camera_get_temperatures_capable', obj.mPtr);
    end

    function ret_val = getTemperatures(obj)
      % getTemperatures
      %   Get the current on camera temperatures
      %
      % Output:
      %   ret_val - vector[numeric] - Temperatures
      %

      ret_val = tof.mex('tof_live_camera_get_temperatures', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getGpio(obj, pin)
      % getGpio
      %   Get voltage level on given pin
      %
      % Inputs:
      %   pin - numeric - Pin
      %
      % Output:
      %   ret_val - logical - Voltage 3.3V
      %

      ret_val = tof.mex('tof_live_camera_get_gpio', obj.mPtr, pin);
    end

    function setGpio(obj, pin, level)
      % setGpio
      %   Set voltage level on given pin
      %
      % Inputs:
      %   pin - numeric - Pin
      %   level - logical - Voltage level
      %

      tof.mex('tof_live_camera_set_gpio', obj.mPtr, pin, level);
    end

  end
end
