import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .camera import *
from .camera_config import *


class TuiCamera(LiveCamera):
    """
    The main interface to the tui camera
    """
    def __del__(self):
        if self._ptr:
            lib.tof_tui_camera_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, serial: Optional[str] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new tui_camera class
        ----------

        Construct the tui camera

        Parameters
        ----------
        serial : str
                The serial number of the camera to connect to, if the camera is
                available over both usb and gige it will be connected over usb. If
                an empty string is supplied the camera will connect to the first usb
                camera, if no usb camera is connected it will connect to the first
                gige camera that replies (this reply time is random, so you might
                connect to a different camera each time if you've got multiple
                cameras connected to your network).
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif serial != None:
            self._ptr = lib.tof_tui_camera_new(serial.encode("utf-8"), get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def setGpioMode(self, pin: int, gpio_mode: GpioMode) -> None:
        """
        Set GPIO mode for a certain pin

        Parameters
        ----------
        pin : int
            Pin
        gpio_mode : GpioMode
            GPIO Mode
        """
        return lib.tof_tui_camera_set_gpio_mode(self._ptr, pin, gpio_mode, get_error())


    def setTriggerMode(self, trigger_mode: TriggerMode) -> None:
        """
        Set trigger mode

        Parameters
        ----------
        trigger_mode : TriggerMode
            Trigger mode
        """
        return lib.tof_tui_camera_set_trigger_mode(self._ptr, trigger_mode, get_error())


def createTuiCameraGige(serial: str, packet_size: int = 1472) -> TuiCamera:
    """
    Construct a tui camera using the gige interface. This could be useful in two
    cases. The first is when the camera is connected with both usb and ethernet but
    you want to use ethernet. The second is when for some reason the default maximum
    packet size is too large.

    Parameters
    ----------
    serial : str
        The serial number of the camera to connect to. If an empty string is
        supplied the camera will connect to the first gige camera that replies (this
        reply time is random, so you might connect to a different camera each time
        if you've got multiple cameras connected to your network).
    packet_size : int
        The packet size to use for communication over ethernet

    Returns
    ----------
    TuiCamera
        A tui camera
    """
    return TuiCamera(ptr = lib.tof_create_tui_camera_gige(serial.encode("utf-8"), packet_size, get_error()))


